#!/usr/bin/perl
#
# Converter for Gauss - Krueger coordinates.
#
# The formulas are based on a similar windows-program.
# Thanks to Norbert Httisch for sending me the neccessary information.
#
# This program is licensed under the GPL.
#
# (c)2000 Andreas Achtzehn (achtzehn@linux-society.de)
#         Nobert Httisch (nobbi@nobbi.com)

$s=@ARGV[0];

if ($s eq "--help") {
	print "
-- Gauss-Converter --
This program converts Gauss-Krueger coordinates into longitude and latitude.

Usage: gauss.pl coordinate [map|maplwp] 
       gauss.pl --help : this help site

	map: Download map of the area using wget
	maplwp: Download map using the LWP::UserAgent
		I recommend to use \"maplwp\". If your perl
		doesn`t support it, try:

		~ perl -MCPAN -e shell
		cpan> install LWP::UserAgent

The coordinate can be either a valid Gauss-Krueger coordinate (14 characters) 
or a VIAG coordinate (12 characters) transmitted on CB 221 within the
German Viag Interkom network. For further details on this transmission
of Gauss-Krueger-coordinates in the German Viag Interkom network
see http://www.heise.de/ct/Redaktion/bo/mobilfunk.

(c)2000 Andreas Achtzehn (achtzehn\@linux-society.de --> 333665574946)
Thanks to Nobert Httisch(nobbi\@nobbi.com) for the sourcecode
of his window\$-program.\n
";
exit(0);
}
if (length($s)==12) {
$sr=substr($s,0,6)*10;
$sx=substr($s,6,6)*10;
print "-- VIAG coordinate --\n";
print "latitude part: $sr\n";
print "longitude part:$sx\n";
}
if (length($s)==14) {
$sr=substr($s,0,7);
$sx=substr($s,7,7);
print "-- Gauss-Krueger coordinate --\n";
print "latitude part: $sr\n";
print "longitude part:$sx\n";

}
if (!(length($s)==14) && !(length($s)==12)) {
print "Invalid coordinate. Coordinate must be either 12 (VIAG) or 14 (GAUSS) characters.\n";
print "See ./gauss.pl --help for details.\n\n";
exit(0);
}

&potsdam;
&modolensky;
if (@ARGV[1] eq "map") {&download;}
if (@ARGV[1] eq "maplwp") {&downloadlwp;}
if (@ARGV[1] eq "") {print "\n\n-- Map has not been downloaded. See gauss.pl --help for details. --\n";} 

sub potsdam {
# Errechnen der Position nach Potsdam-Koordinatensystem
$bm  = int($sr/1000000);
$y   = $sr-($bm*1000000+500000);
$si  = $sx/111120.6196;
$px  = $si+0.143885358*sin(2*$si*0.017453292)+0.00021079*sin(4*$si*0.017453292)+0.000000423*sin(6*$si*0.017453292);
$t   = (sin($px*0.017453292))/(cos($px*0.017453292));
$v   = sqrt(1+0.006719219*cos($px*0.017453292)*cos($px*0.017453292));
$ys  = ($y*$v)/6398786.85;
$lat = $px-$ys*$ys*57.29577*$t*$v*$v*(0.5-$ys*$ys*(4.97-3*$t*$t)/24);
$dl  = $ys*57.29577/cos($px*0.017453292) * (1-$ys*$ys/6*($v*$v+2*$t*$t-$ys*$ys*(0.6+1.1*$t*$t)*(0.6+1.1*$t*$t)));
$lon = $bm*3+$dl;

$ph =int($lat);
$pz =($lat-$ph)*60;
$pm =int($pz);
$ps =($pz-$pm)*60;

$lh =int($lon);
$lz =($lon-$lh)*60;
$lm =int($lz);
$ls =($lz-$lm)*60;
 print "-- Coordinate in German Potsdam system --\n";
 print "Latitude: $ph  $pm ' $ps \" ";
 print "($lat)\n";
 print "Longitude: $lh  $lm ' $ls \" ";
 print "($lon)\n";
}
sub modolensky {
$potsd_a  = 6377397.155;
$wgs84_a  = 6378137.0;
$potsd_f  = 1/299.152812838;
$wgs84_f  = 1/298.257223563;

$potsd_es = 2*$potsd_f - $potsd_f*$potsd_f;

$potsd_dx = 606.0;
$potsd_dy = 23.0;
$potsd_dz = 413.0;
$pi = 3.141592654;
$latr = $lat/180*$pi;
$lonr = $lon/180*$pi;

$sa = sin($latr);
$ca = cos($latr);
$so = sin($lonr);
$co = cos($lonr);

$bda  = 1-$potsd_f;

$delta_a = $wgs84_a - $potsd_a;
$delta_f = $wgs84_f - $potsd_f;

$rn = $potsd_a / sqrt(1-$potsd_es*sin($latr)*sin($latr));
$rm = $potsd_a * ((1-$potsd_es)/sqrt(1-$potsd_es*sin($latr)*sin($latr)*1-$potsd_es*sin($latr)*sin($latr)*1-$potsd_es*sin($latr)*sin($latr)));

$ta = (-$potsd_dx*$sa*$co - $potsd_dy*$sa*$so)+$potsd_dz*$ca;
$tb = $delta_a*(($rn*$potsd_es*$sa*$ca)/$potsd_a);
$tc = $delta_f*($rm/$bda+$rn*$bda)*$sa*$ca;
$dlat = ($ta+$tb+$tc)/$rm;

$dlon = (-$potsd_dx*$so + $potsd_dy*$co)/($rn*$ca);

$wgs84lat = ($latr + $dlat)*180/$pi;
$wgs84lon = ($lonr + $dlon)*180/$pi;
$ph=int($wgs84lat);
$pz=($wgs84lat-$ph)*60;
$pm=int($pz);
$ps=int(($pz-$pm)*60);

$lh=int($wgs84lon);
$lz=($wgs84lon-$lh)*60;
$lm=int($lz);
$ls=int(($lz-$lm)*60);
 print "\n\n-- Coordinate in WGS84 system --\n";
 print "Latitude: $ph  $pm ' $ps \" ";
 print "($wgs84lat)\n";
 print "Longitude: $lh  $lm ' $ls \" ";
 print "($wgs84lon)\n";

}
sub download {
	if ($wgs84lat > 10) { $wgs84lat = substr($wgs84lat,0,6); } else { $wgs84lat = substr($wgs84lat,0,5); }
	if ($wgs84lon > 10) { $wgs84lon = substr($wgs84lon,0,6); } else { $wgs84lon = substr($wgs84lon,0,5); }

	$bef = "wget -q \"http://www.mapblast.com/gif?&CT=$wgs84lat:$wgs84lon:20000&IC=$wgs84lat:$wgs84lon:8:&W=456&H=259&FAM=myblast&LB=\"; mv gif* map.gif";
	system ($bef);
	print "\n\n-- Map downloaded and saved to map.gif --\n"
}
sub downloadlwp {
	if ($wgs84lat > 10) { $wgs84lat = substr($wgs84lat,0,6); } else { $wgs84lat = substr($wgs84lat,0,5); }
	if ($wgs84lon > 10) { $wgs84lon = substr($wgs84lon,0,6); } else { $wgs84lon = substr($wgs84lon,0,5); }


        require LWP::UserAgent;
        $ua = new LWP::UserAgent;
	$anfrage = "http://www.mapblast.com/gif?&CT=".$wgs84lat.":".$wgs84lon.":20000&IC=".$wgs84lat.":".$wgs84lon.":8:&W=456&H=259&FAM=myblast&LB=";       
        $request = new HTTP::Request('GET',$anfrage);
       
        $response = $ua->request($request, 'map.gif');
	print "\n\n-- Map downloaded and saved to map.gif --\n"
       
}
